package net.wanji.web.service.impl;

import lombok.extern.slf4j.Slf4j;
import net.wanji.feign.pojo.result.JsonViewObject;
import net.wanji.feign.pojo.vo.*;
import net.wanji.feign.service.UtcFeignClients;
import net.wanji.web.mapper.ControlHistMapper;
import net.wanji.web.service.FeignProxyService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.Objects;

/**
 * @author duanruiming
 * @date 2023/02/09 10:44
 */
@Service
@Slf4j
@SuppressWarnings("all")
public class FeignProxyServiceImpl implements FeignProxyService {

    @Autowired
    private UtcFeignClients utcFeignClients;
    @Autowired
    private ControlHistMapper controlHistMapper;

    @Override
    public JsonViewObject schemeSend(SchemeSendVO schemeSendVO) {
        JsonViewObject jsonViewObject = utcFeignClients.schemeSend(schemeSendVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("信号机方案下发-基础方案下发UTC服务调用异常");
        }
        return jsonViewObject.success("信号机方案下发成功");
    }

    @Override
    public JsonViewObject planSend(PlanSendVO planSendVO) {
        JsonViewObject jsonViewObject = utcFeignClients.planSend(planSendVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("计划下发-计划信息、时段信息UTC服务调用异常");
        }
        return jsonViewObject.success("计划下发-计划信息、时段信息下发成功");
    }

    @Override
    public JsonViewObject scheduleSend(ScheduleSendVO scheduleSendVO) {
        JsonViewObject jsonViewObject = utcFeignClients.scheduleSend(scheduleSendVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("时间表下发-时间表信息UTC服务调用异常");
        }
        return jsonViewObject.success("时间表下发-时间表信息下发成功");
    }

    @Override
    public JsonViewObject phaseTimingSend(PhaseTimingSendVO phaseTimingSendVO) {
        JsonViewObject jsonViewObject = utcFeignClients.phaseTimingSend(phaseTimingSendVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("相位配时下发-（下发相位参数）UTC服务调用异常");
        }
        return jsonViewObject.success("相位配时下发-相位参数下发成功");
    }

    @Override
    public JsonViewObject allRedControl(ControlCommandVO commandVO) {
        Integer controlType = commandVO.getControlType();
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        if (1 == controlType) {
            jsonViewObject = utcFeignClients.allRedControl(commandVO);
        }
        if (2 == controlType) {
            jsonViewObject = utcFeignClients.yellowLightControl(commandVO);
        }
        if (3 == controlType) {
            jsonViewObject = utcFeignClients.closeLightControl(commandVO);
        }
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("灯态控制UTC服务调用异常");
        }
        insertControlHist(commandVO.getCrossCode(), controlType + 1, "admin");
        return jsonViewObject.success("特殊灯态控制下发成功");
    }

    @Override
    public JsonViewObject stepControl(StepControlVO stepControlVO) {
        JsonViewObject jsonViewObject = utcFeignClients.stepControl(stepControlVO.getCode(), stepControlVO.getCommand(), stepControlVO.getStepNum());
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("步进控制-步进控制/恢复UTC服务调用异常");
        }
        insertControlHist(stepControlVO.getCode(), 7, "admin");
        return jsonViewObject.success("步进控制下发成功");
    }

    @Override
    public JsonViewObject recoverSchedule(String code) throws Exception {
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();
        jsonViewObject = utcFeignClients.recoverSchedule(code);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("恢复时间表-恢复路口时间表执行UTC服务调用异常");
        }
        insertControlHist(code, 1, "admin");
        return jsonViewObject.success("恢复时间表-恢复路口时间表下发成功");
    }

    @Override
    public JsonViewObject lockControl(LockControlVO lockControlVO) {
        JsonViewObject jsonViewObject = utcFeignClients.lockControl(lockControlVO);
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("锁定控制-相位锁定/解锁UTC服务调用异常");
        }
        insertControlHist(lockControlVO.getCrossCode(), 4, "admin");
        return jsonViewObject.success("锁定控制-相位锁定/解锁下发成功");
    }

    @Override
    public JsonViewObject phaseDiffSend(HttpServletRequest httpServletRequest) {
        JsonViewObject jsonViewObject = utcFeignClients.phaseDiffSend();
        if (Objects.isNull(jsonViewObject) || jsonViewObject.getCode() != 200) {
            return jsonViewObject.fail("相位差下发UTC服务调用异常");
        }
        return jsonViewObject.success("相位差下发成功");
    }

    /**
     * 特殊控制插入本地控制历史表
     *
     * @param crossCode
     * @param type
     * @param userName
     */
    private void insertControlHist(String crossCode, Integer type, String userName) {
        controlHistMapper.insertOne(crossCode, type, userName);
    }
}
