package net.wanji.utc.controller;

import com.github.pagehelper.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import net.wanji.utc.dto.systemadmin.CrossInfoInsertOrUpdateDTO;
import net.wanji.utc.dto.systemadmin.CrossInfoListDTO;
import net.wanji.utc.dto.systemadmin.DeleteByStringIdListDTO;
import net.wanji.utc.entity.JsonViewObject;
import net.wanji.utc.service.systemadmin.CrossInfoService;
import net.wanji.utc.vo.systemadmin.CrossInfoListVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.ws.rs.core.MediaType;
import java.util.Objects;

/**
 * 系统管理-信号机管理
 *
 * @author Kent HAN
 * @date 2022/11/24 8:32
 */

@Api(value = "系统管理-信号机管理", description = "系统管理-信号机管理")
@RequestMapping("/crossInfo")
@RestController
public class CrossInfoController {
    @Autowired
    CrossInfoService crossInfoService;

    @ApiOperation(value = "信号机列表", notes = "信号机列表", response = CrossInfoListVO.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/list",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = CrossInfoListVO.class),
    })
    public JsonViewObject list(@RequestBody CrossInfoListDTO crossInfoListDTO) {
        Integer pageNum = crossInfoListDTO.getPageNum();
        Integer pageSize = crossInfoListDTO.getPageSize();
        // 不分页
        if (Objects.isNull(pageNum) || pageNum <= 0) {
            crossInfoListDTO.setPageNum(1);
        }
        if (Objects.isNull(pageSize) || pageSize <= 0) {
            crossInfoListDTO.setPageSize(9999);
        }
        PageInfo<CrossInfoListVO> crossInfoListOutVOPageInfo = crossInfoService.list(crossInfoListDTO);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(crossInfoListOutVOPageInfo);
    }

    @ApiOperation(value = "信号机添加或修改", notes = "信号机添加或修改", consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/insertOrUpdate", consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject insertOrUpdate(@RequestBody CrossInfoInsertOrUpdateDTO inVO) {
        crossInfoService.insertOrUpdate(inVO);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success();
    }

    @ApiOperation(value = "信号机删除", notes = "信号机删除", consumes = MediaType.APPLICATION_JSON)
    @PostMapping(value = "/delete", consumes = MediaType.APPLICATION_JSON)
    public JsonViewObject delete(@RequestBody DeleteByStringIdListDTO inVO) {
        crossInfoService.delete(inVO);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success();
    }
}
