package net.wanji.utc.controller;

import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import net.wanji.utc.entity.JsonViewObject;
import net.wanji.utc.po.*;
import net.wanji.utc.service.staticinfo.StaticInfoService;
import net.wanji.utc.dto.CrossInfoDTO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import net.wanji.utc.dto.CrossSchedulesDTO;
import net.wanji.utc.dto.PlanSectionDTO;
import net.wanji.utc.dto.SchemePhaseLightsDTO;
import net.wanji.utc.vo.PlanSectionVO;
import net.wanji.utc.vo.SchemePhaseLightsVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.ws.rs.core.MediaType;
import java.util.List;

/**
 * 静态信息接口
 *
 * @date 2022/11/15 9:38
 */
@Api(value = "静态信息接口", description = "静态信息接口")
@RequestMapping("/staticInfo")
@RestController
public class StaticInfoController {
    @Autowired
    StaticInfoService staticInfoService;

    @PostMapping(value = "/crossInfo",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiOperation(value = "信号路口基础信息", notes = "信号路口基础信息", response = CrossInfoPO.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = CrossInfoPO.class)
    })
    public JsonViewObject crossInfo(@RequestBody CrossInfoDTO crossInfoDTO) throws Exception {
        List<CrossInfoPO> crossInfoPOList = staticInfoService.crossBasicInfo(crossInfoDTO);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(crossInfoPOList);
    }

    @PostMapping(value = "/schemePhaseLights",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiOperation(value = "方案数据-方案信息、相位信息、灯组信息", notes = "方案数据-方案信息、相位信息、灯组信息",
            response = SchemePhaseLightsVO.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = SchemePhaseLightsVO.class)
    })
    public JsonViewObject schemePhaseLights(@RequestBody SchemePhaseLightsDTO schemePhaseLightsDTO) throws Exception {
        // 更新数据库
        staticInfoService.schemePhaseLights(schemePhaseLightsDTO);
        // 构造返回值
        SchemePhaseLightsVO schemePhaseLightsVO = staticInfoService.buildSchemePhaseLightsResponse(schemePhaseLightsDTO);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(schemePhaseLightsVO);
    }

    @PostMapping(value = "/planSection",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiOperation(value = "计划数据-计划信息、时段信息", notes = "计划数据-计划信息、时段信息", response = PlanSectionVO.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = PlanSectionVO.class)
    })
    public JsonViewObject planSection(@RequestBody PlanSectionDTO planSectionDTO) throws Exception {
        // 更新数据库
        staticInfoService.planSection(planSectionDTO);
        // 构造返回值
        String crossId = planSectionDTO.getCrossId();
        Integer planNo = planSectionDTO.getPlanNo();
        PlanSectionVO planSectionVO = new PlanSectionVO();
        if (planNo == -1) {
            // 返回所有
            planSectionVO = staticInfoService.buildPlanSectionResponse(crossId);
        } else {
            // 按计划号返回
            planSectionVO = staticInfoService.buildPlanSectionResponse(crossId, planNo);
        }
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(planSectionVO);
    }

    @PostMapping(value = "/crossSchedules",
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiOperation(value = "时间表数据", notes = "时间表数据", response = CrossSchedulesPO.class,
            produces = MediaType.APPLICATION_JSON, consumes = MediaType.APPLICATION_JSON)
    @ApiResponses({
            @ApiResponse(code = 200, message = "OK", response = CrossSchedulesPO.class)
    })
    public JsonViewObject crossSchedules(@RequestBody CrossSchedulesDTO crossSchedulesDTO) throws Exception {
        // 更新数据库
        staticInfoService.crossSchedules(crossSchedulesDTO);
        // 构造返回值
        List<String> crossIdList = crossSchedulesDTO.getCrossIdList();
        List<CrossSchedulesPO> crossSchedulesPOList = staticInfoService.buildCrossSchedulesResponse(crossIdList);
        JsonViewObject jsonViewObject = JsonViewObject.newInstance();

        return jsonViewObject.success(crossSchedulesPOList);
    }
}
