package net.wanji.utc.service.staticinfo.impl;

import lombok.RequiredArgsConstructor;
import net.wanji.utc.common.typeenum.BasicEnum;
import net.wanji.utc.dto.CrossInfoDTO;
import net.wanji.utc.dto.CrossSchedulesDTO;
import net.wanji.utc.dto.PlanSectionDTO;
import net.wanji.utc.dto.SchemePhaseLightsDTO;
import net.wanji.utc.mapper.*;
import net.wanji.utc.po.*;
import net.wanji.utc.service.staticinfo.WanJiCommonStaticInfoService;
import net.wanji.utc.service.staticinfo.*;
import net.wanji.utc.vo.PlanSectionVO;
import net.wanji.utc.vo.SchemePhaseLightsVO;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * @author Kent HAN
 * @date 2022/11/15 10:32
 */
@Service
@RequiredArgsConstructor
public class StaticInfoServiceImpl implements StaticInfoService {
    private final HkCrossInfoService hkCrossInfoService;
    private final CrossInfoMapper crossInfoMapper;
    private final ManufacturerInfoMapper manufacturerInfoMapper;
    private final HkSchemePhaseLightsService hkSchemePhaseLightsService;
    private final HkPlanSectionService hkPlanSectionService;
    private final HkCrossSchedulesService hkCrossSchedulesService;
    private final CrossSchemeMapper crossSchemeMapper;
    private final CrossPhaseMapper crossPhaseMapper;
    private final CrossLightsMapper crossLightsMapper;
    private final CrossPhaseLightsMapper crossPhaseLightsMapper;
    private final CrossPlanMapper crossPlanMapper;
    private final CrossSectionMapper crossSectionMapper;
    private final CrossSchedulesMapper crossSchedulesMapper;
    private final WanJiCommonStaticInfoService wanjiCommonStaticInfoService;

    @Override
    @Transactional
    public List<CrossInfoPO> crossBasicInfo(CrossInfoDTO crossInfoDTO) throws Exception {
        List<CrossInfoPO> crossInfoPOList = new ArrayList<>();
        if (Objects.equals(BasicEnum.ManufacturerEnum.HK.getCode(), crossInfoDTO.getManufacturerCode())) {
            crossInfoPOList = hkCrossInfoService.hkCrossBasicInfo();
        } else {
            // todo 其他厂商

        }
        //存储信号机信息
        if (crossInfoPOList.size() == 0) {
            return null;
        }
        for (CrossInfoPO crossInfoPO : crossInfoPOList) {
            String code = crossInfoPO.getCode();
            Integer manufacturerId = crossInfoPO.getManufacturerId();
            String id = crossInfoMapper.selectIdByCodeAndManufacturerId(code, manufacturerId);
            if (id != null) {
                // 如果有记录，则更新
                crossInfoMapper.updateOneByCodeAndManufacturerId(crossInfoPO);
            } else {
                // 如果无记录，则新增
                crossInfoMapper.insertOne(crossInfoPO);
            }
        }
        String manufacturerCode = crossInfoDTO.getManufacturerCode();
        ManufacturerInfoPO manufacturerInfoPO = manufacturerInfoMapper.selectByCode(manufacturerCode);
        Integer manufacturerId = manufacturerInfoPO.getId();
        crossInfoPOList = crossInfoMapper.selectByManufacturerId(manufacturerId);
        // 释放内存
        hkCrossInfoService.clearRes();
        return crossInfoPOList;
    }

    @Override
    @Transactional
    public void schemePhaseLights(SchemePhaseLightsDTO schemePhaseLightsDTO) throws Exception {
        String crossId = schemePhaseLightsDTO.getCrossId();
        // 获取厂商ID和信号机ID
        CrossInfoPO crossInfoPO = crossInfoMapper.selectByPrimaryKey(crossId);
        String crossCode = crossInfoPO.getCode();
        Integer manufacturerId = crossInfoPO.getManufacturerId();
        // 获取厂商代码
        ManufacturerInfoPO manufacturerInfoPO = manufacturerInfoMapper.selectById(manufacturerId);
        String manufacturerCode = manufacturerInfoPO.getCode();
        if (Objects.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerCode)) {
            // 海康
            hkSchemePhaseLightsService.hkSchemePhaseLights(crossId, crossCode);
            // todo 其他厂商
        } else {
            // 默认万集标准服务
            wanjiCommonStaticInfoService.schemePhaseLights(crossId, crossCode);
        }
    }

    @Override
    @Transactional
    public void planSection(PlanSectionDTO planSectionDTO) throws Exception {
        String crossId = planSectionDTO.getCrossId();
        // 获取厂商ID和信号机ID
        CrossInfoPO crossInfoPO = crossInfoMapper.selectByPrimaryKey(crossId);
        String crossCode = crossInfoPO.getCode();
        String crossInfoId = crossInfoPO.getId();
        Integer manufacturerId = crossInfoPO.getManufacturerId();
        // 获取厂商代码
        ManufacturerInfoPO manufacturerInfoPO = manufacturerInfoMapper.selectById(manufacturerId);
        String manufacturerCode = manufacturerInfoPO.getCode();
        Integer planNo = planSectionDTO.getPlanNo();
        if (Objects.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerCode)) {
            // 海康
            hkPlanSectionService.planSection(crossCode, planNo, crossId);
        } else {
            wanjiCommonStaticInfoService.planSection(crossInfoId, crossCode, planNo);
        }
    }

    @Override
    @Transactional
    public void crossSchedules(CrossSchedulesDTO crossSchedulesDTO) throws Exception {
        String manufacturerCode = crossSchedulesDTO.getManufacturerCode();
        List<String> crossIdList = crossSchedulesDTO.getCrossIdList();
        if (Objects.equals(BasicEnum.ManufacturerEnum.HK.getCode(), manufacturerCode)) {
            // 海康
            hkCrossSchedulesService.crossSchedules(crossIdList);
        } else {
            if (!CollectionUtils.isEmpty(crossIdList)) {
                String crossId = crossIdList.get(0);
                CrossInfoPO crossInfoPO = crossInfoMapper.selectByPrimaryKey(crossId);
                String signalIc = crossInfoPO.getCode();
                wanjiCommonStaticInfoService.schedules(crossId, signalIc);
            }
        }

    }

    @Override
    public SchemePhaseLightsVO buildSchemePhaseLightsResponse(SchemePhaseLightsDTO schemePhaseLightsDTO) {
        String crossId = schemePhaseLightsDTO.getCrossId();
        List<CrossSchemePO> crossSchemePOList = crossSchemeMapper.selectByCrossId(crossId);
        List<CrossPhasePO> crossPhasePOList = crossPhaseMapper.selectByCrossId(crossId);
        List<CrossLightsPO> crossLightsPOList = crossLightsMapper.selectByCrossId(crossId);
        List<CrossPhaseLightsPO> crossPhaseLightsPOList = crossPhaseLightsMapper.selectByCrossId(crossId);


        SchemePhaseLightsVO schemePhaseLightsVO = new SchemePhaseLightsVO();
        schemePhaseLightsVO.setCrossSchemeList(crossSchemePOList);
        schemePhaseLightsVO.setCrossPhaseList(crossPhasePOList);
        schemePhaseLightsVO.setCrossLightsList(crossLightsPOList);
        schemePhaseLightsVO.setCrossPhaseLightsPOList(crossPhaseLightsPOList);

        return schemePhaseLightsVO;
    }

    @Override
    public PlanSectionVO buildPlanSectionResponse(String crossId) {
        List<CrossPlanPO> crossPlanPOList = crossPlanMapper.selectByCrossId(crossId);
        List<CrossSectionPO> crossSectionPOList = crossSectionMapper.selectByCrossId(crossId);

        PlanSectionVO planSectionVO = new PlanSectionVO();
        planSectionVO.setCrossPlanPOList(crossPlanPOList);
        planSectionVO.setCrossSectionPOList(crossSectionPOList);

        return planSectionVO;
    }

    @Override
    public PlanSectionVO buildPlanSectionResponse(String crossId, Integer planNo) {
        List<CrossPlanPO> crossPlanPOList = crossPlanMapper.selectByCrossIdAndPlanNo(crossId, planNo);
        Integer planId = crossPlanMapper.selectIdByNo(crossId, planNo);
        List<CrossSectionPO> crossSectionPOList = crossSectionMapper.selectByCrossIdAndPlanId(crossId, planId);

        PlanSectionVO planSectionVO = new PlanSectionVO();
        planSectionVO.setCrossPlanPOList(crossPlanPOList);
        planSectionVO.setCrossSectionPOList(crossSectionPOList);

        return planSectionVO;
    }

    @Override
    public List<CrossSchedulesPO> buildCrossSchedulesResponse(List<String> crossIdList) {
        List<CrossSchedulesPO> crossSchedulesPOList = crossSchedulesMapper.selectByCrossIds(crossIdList);
        return crossSchedulesPOList;
    }
}
