package net.wanji.utc.task;

import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.SerializerFeature;
import lombok.extern.slf4j.Slf4j;
import net.wanji.utc.po.SignalStatusLogPO;
import net.wanji.utc.service.runninginfo.SignalStatusService;
import net.wanji.utc.vo.LightsStatusVO;
import net.wanji.utc.websocket.RealTimeDataWebSocket;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

/**
 * 订阅信号运行状态、告警、灯态
 *
 * @author Kent HAN
 * @date 2022/11/21 9:01
 */
@Component
@Slf4j
@SuppressWarnings("all")
public class SignalStatusTask {
    @Autowired
    SignalStatusService signalStatusService;

    // 运行状态、告警，5分钟一次
    @Scheduled(fixedRate = 1 * 60 * 1000)
    public void runningStatusAlarm() {
        List<SignalStatusLogPO> signalStatusLogPOS = signalStatusService.runningStatusAlarm();
    }

    // 灯态，5秒一次
    @Scheduled(fixedRate = 1 * 1000)
    public void lightStatus() {
        try {

            ConcurrentHashMap<String, RealTimeDataWebSocket> evenWarnWebSocketMap = RealTimeDataWebSocket.getEvenWarnWebSocketMap();
            if (!evenWarnWebSocketMap.isEmpty()) {
                for (Map.Entry<String, RealTimeDataWebSocket> entry : evenWarnWebSocketMap.entrySet()) {
                    String crossId = entry.getKey();
                    RealTimeDataWebSocket webSocket = entry.getValue();
                    List<LightsStatusVO> lightsStatusVOS = signalStatusService.lightStatus();
                    webSocket.sendInfo(JSONObject.toJSONString(lightsStatusVOS, SerializerFeature.WriteMapNullValue), crossId);
                }
            }

        } catch (Exception e) {
            log.error("实时推送灯态数据失败", e);
        }
    }
}
